@@ -29,29 +29,30 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: chfn.c,v 1.29 2005/07/07 15:32:50 kloczek Exp $")
-#include <sys/types.h>
-#include <stdio.h>
+#ident "$Id: chfn.c,v 1.38 2005/10/19 15:21:07 kloczek Exp $"
+
 #include <fcntl.h>
-#include <signal.h>
-#include "prototypes.h"
-#include "defines.h"
 #include <pwd.h>
-#include "pwio.h"
-#include "getdef.h"
-#include "pwauth.h"
-#include "nscd.h"
+#include <signal.h>
+#include <stdio.h>
+#include <sys/types.h>
 #ifdef HAVE_SHADOW_H
 #include <shadow.h>
 #endif
-#ifdef USE_PAM
-#include "pam_defs.h"
-#endif
 #ifdef WITH_SELINUX
 #include <selinux/selinux.h>
 #include <selinux/av_permissions.h>
 #endif
+#include "defines.h"
+#include "exitcodes.h"
+#include "getdef.h"
+#include "nscd.h"
+#ifdef USE_PAM
+#include "pam_defs.h"
+#endif
+#include "prototypes.h"
+#include "pwauth.h"
+#include "pwio.h"
 /*
  * Global variables.
  */
@@ -76,7 +77,6 @@ static char *copy_field (char *, char *, char *);
 /*
  * usage - print command line syntax and exit
  */
-
 static void usage (void)
 {
 	if (amroot)
@@ -88,7 +88,7 @@ static void usage (void)
 		fprintf (stderr,
 			 _("Usage: %s [-f full_name] [-r room_no] "
 			   "[-w work_ph] [-h home_ph]\n"), Prog);
-	exit (1);
+	exit (E_USAGE);
 }
 
 
@@ -133,7 +133,6 @@ static int may_change_field (int field)
  * prompt the user for each of the four fields and fill in the fields from
  * the user's response, or leave alone if nothing was entered.
  */
-
 static void new_fields (void)
 {
 	printf (_("Enter the new value, or press ENTER for the default\n"));
@@ -146,17 +145,17 @@ static void new_fields (void)
 	if (may_change_field ('r'))
 		change_field (roomno, sizeof roomno, _("Room Number"));
 	else
-		printf ("\t%s: %s\n", _("Room Number"), roomno);
+		printf (_("\tRoom Number: %s\n"), roomno);
 
 	if (may_change_field ('w'))
 		change_field (workph, sizeof workph, _("Work Phone"));
 	else
-		printf ("\t%s: %s\n", _("Work Phone"), workph);
+		printf (_("\tWork Phone: %s\n"), workph);
 
 	if (may_change_field ('h'))
 		change_field (homeph, sizeof homeph, _("Home Phone"));
 	else
-		printf ("\t%s: %s\n", _("Home Phone"), homeph);
+		printf (_("\tHome Phone: %s\n"), homeph);
 
 	if (amroot)
 		change_field (slop, sizeof slop, _("Other"));
@@ -172,7 +171,6 @@ static void new_fields (void)
  *	out - where to copy the field to
  *	extra - fields with '=' get copied here
  */
-
 static char *copy_field (char *in, char *out, char *extra)
 {
 	char *cp = NULL;
@@ -198,7 +196,6 @@ static char *copy_field (char *in, char *out, char *extra)
 	return cp;
 }
 
-
 /*
  * chfn - change a user's password file information
  *
@@ -215,7 +212,6 @@ static char *copy_field (char *in, char *out, char *extra)
  *
  *	(*) requires root permission to execute.
  */
-
 int main (int argc, char **argv)
 {
 	char *cp;		/* temporary character pointer       */
@@ -231,6 +227,12 @@ int main (int argc, char **argv)
 	int oflg = 0;		/* -o - set other information        */
 	char *user;
 
+#ifdef USE_PAM
+	pam_handle_t *pamh = NULL;
+	struct passwd *pampw;
+	int retval;
+#endif
+
 	sanitize_env ();
 	setlocale (LC_ALL, "");
 	bindtextdomain (PACKAGE, LOCALEDIR);
@@ -240,14 +242,12 @@ int main (int argc, char **argv)
 	 * This command behaves different for root and non-root
 	 * users.
 	 */
-
 	amroot = (getuid () == 0);
 
 	/*
 	 * Get the program name. The program name is used as a
 	 * prefix to most error messages.
 	 */
-
 	Prog = Basename (argv[0]);
 
 	OPENLOG ("chfn");
@@ -259,14 +259,13 @@ int main (int argc, char **argv)
 	 * environment and must agree with the real UID. Also, the UID will
 	 * be checked for any commands which are restricted to root only.
 	 */
-
 	while ((flag = getopt (argc, argv, "f:r:w:h:o:")) != EOF) {
 		switch (flag) {
 		case 'f':
 			if (!may_change_field ('f')) {
 				fprintf (stderr,
 					 _("%s: Permission denied.\n"), Prog);
-				exit (1);
+				exit (E_NOPERM);
 			}
 			fflg++;
 			STRFCPY (fullnm, optarg);
@@ -275,7 +274,7 @@ int main (int argc, char **argv)
 			if (!may_change_field ('h')) {
 				fprintf (stderr,
 					 _("%s: Permission denied.\n"), Prog);
-				exit (1);
+				exit (E_NOPERM);
 			}
 			hflg++;
 			STRFCPY (homeph, optarg);
@@ -284,7 +283,7 @@ int main (int argc, char **argv)
 			if (!may_change_field ('r')) {
 				fprintf (stderr,
 					 _("%s: Permission denied.\n"), Prog);
-				exit (1);
+				exit (E_NOPERM);
 			}
 			rflg++;
 			STRFCPY (roomno, optarg);
@@ -293,7 +292,7 @@ int main (int argc, char **argv)
 			if (!amroot) {
 				fprintf (stderr,
 					 _("%s: Permission denied.\n"), Prog);
-				exit (1);
+				exit (E_NOPERM);
 			}
 			oflg++;
 			STRFCPY (slop, optarg);
@@ -302,7 +301,7 @@ int main (int argc, char **argv)
 			if (!may_change_field ('w')) {
 				fprintf (stderr,
 					 _("%s: Permission denied.\n"), Prog);
-				exit (1);
+				exit (E_NOPERM);
 			}
 			wflg++;
 			STRFCPY (workph, optarg);
@@ -316,14 +315,13 @@ int main (int argc, char **argv)
 	 * Get the name of the user to check. It is either the command line
 	 * name, or the name getlogin() returns.
 	 */
-
 	if (optind < argc) {
 		user = argv[optind];
 		pw = getpwnam (user);
 		if (!pw) {
 			fprintf (stderr, _("%s: unknown user %s\n"), Prog,
 				 user);
-			exit (1);
+			exit (E_NOPERM);
 		}
 	} else {
 		pw = get_my_pwent ();
@@ -332,7 +330,7 @@ int main (int argc, char **argv)
 				 _
 				 ("%s: Cannot determine your user name.\n"),
 				 Prog);
-			exit (1);
+			exit (E_NOPERM);
 		}
 		user = xstrdup (pw->pw_name);
 	}
@@ -341,7 +339,6 @@ int main (int argc, char **argv)
 	/*
 	 * Now we make sure this is a LOCAL password entry for this user ...
 	 */
-
 	if (__ispwNIS ()) {
 		char *nis_domain;
 		char *nis_master;
@@ -357,7 +354,7 @@ int main (int argc, char **argv)
 				 ("%s: `%s' is the NIS master for this client.\n"),
 				 Prog, nis_master);
 		}
-		exit (1);
+		exit (E_NOPERM);
 	}
 #endif
 
@@ -365,23 +362,21 @@ int main (int argc, char **argv)
 	 * Non-privileged users are only allowed to change the gecos field
 	 * if the UID of the user matches the current real UID.
 	 */
-
 	if (!amroot && pw->pw_uid != getuid ()) {
 		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 #ifdef WITH_SELINUX
 	/*
 	 * If the UID of the user does not match the current real UID,
 	 * check if the change is allowed by SELinux policy.
 	 */
-
 	if ((pw->pw_uid != getuid ())
-	    && (checkPasswdAccess (PASSWD__CHFN) != 0)) {
+	    && (selinux_check_passwd_access (PASSWD__CHFN) != 0)) {
 		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 #endif
 
@@ -392,17 +387,45 @@ int main (int argc, char **argv)
 	 * any changes can be made. Idea from util-linux chfn/chsh. 
 	 * --marekm
 	 */
-
 	if (!amroot && getdef_bool ("CHFN_AUTH"))
 		passwd_check (pw->pw_name, pw->pw_passwd, "chfn");
 
-#endif				/* !USE_PAM */
+#else				/* !USE_PAM */
+	retval = PAM_SUCCESS;
+
+	pampw = getpwuid (getuid ());
+	if (pampw == NULL) {
+		retval = PAM_USER_UNKNOWN;
+	}
+
+	if (retval == PAM_SUCCESS) {
+		retval = pam_start ("chfn", pampw->pw_name, &conv, &pamh);
+	}
+
+	if (retval == PAM_SUCCESS) {
+		retval = pam_authenticate (pamh, 0);
+		if (retval != PAM_SUCCESS) {
+			pam_end (pamh, retval);
+		}
+	}
+
+	if (retval == PAM_SUCCESS) {
+		retval = pam_acct_mgmt (pamh, 0);
+		if (retval != PAM_SUCCESS) {
+			pam_end (pamh, retval);
+		}
+	}
+
+	if (retval != PAM_SUCCESS) {
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
+		exit (E_NOPERM);
+	}
+#endif				/* USE_PAM */
 
 	/*
 	 * Now get the full name. It is the first comma separated field in
 	 * the GECOS field.
 	 */
-
 	STRFCPY (old_gecos, pw->pw_gecos);
 	cp = copy_field (old_gecos, fflg ? (char *) 0 : fullnm, slop);
 
@@ -410,21 +433,18 @@ int main (int argc, char **argv)
 	 * Now get the room number. It is the next comma separated field,
 	 * if there is indeed one.
 	 */
-
 	if (cp)
 		cp = copy_field (cp, rflg ? (char *) 0 : roomno, slop);
 
 	/*
 	 * Now get the work phone number. It is the third field.
 	 */
-
 	if (cp)
 		cp = copy_field (cp, wflg ? (char *) 0 : workph, slop);
 
 	/*
 	 * Now get the home phone number. It is the fourth field.
 	 */
-
 	if (cp)
 		cp = copy_field (cp, hflg ? (char *) 0 : homeph, slop);
 
@@ -453,44 +473,43 @@ int main (int argc, char **argv)
 	if (valid_field (fullnm, ":,=")) {
 		fprintf (stderr, _("%s: invalid name: \"%s\"\n"), Prog, fullnm);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (valid_field (roomno, ":,=")) {
 		fprintf (stderr, _("%s: invalid room number: \"%s\"\n"),
 			 Prog, roomno);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (valid_field (workph, ":,=")) {
 		fprintf (stderr, _("%s: invalid work phone: \"%s\"\n"),
 			 Prog, workph);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (valid_field (homeph, ":,=")) {
 		fprintf (stderr, _("%s: invalid home phone: \"%s\"\n"),
 			 Prog, homeph);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (valid_field (slop, ":")) {
 		fprintf (stderr,
 			 _("%s: \"%s\" contains illegal characters\n"),
 			 Prog, slop);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	/*
 	 * Build the new GECOS field by plastering all the pieces together,
 	 * if they will fit ...
 	 */
-
 	if (strlen (fullnm) + strlen (roomno) + strlen (workph) +
 	    strlen (homeph) + strlen (slop) > (unsigned int) 80) {
 		fprintf (stderr, _("%s: fields too long\n"), Prog);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	snprintf (new_gecos, sizeof new_gecos, "%s,%s,%s,%s%s%s",
 		  fullnm, roomno, workph, homeph, slop[0] ? "," : "", slop);
@@ -501,12 +520,11 @@ int main (int argc, char **argv)
 	 * against unexpected signals. Any keyboard signals are set to be
 	 * ignored.
 	 */
-
 	if (setuid (0)) {
 		fprintf (stderr, _("Cannot change ID to root.\n"));
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	pwd_init ();
 
@@ -514,21 +532,20 @@ int main (int argc, char **argv)
 	 * The passwd entry is now ready to be committed back to the
 	 * password file. Get a lock on the file and open it.
 	 */
-
 	if (!pw_lock ()) {
 		fprintf (stderr,
 			 _
 			 ("Cannot lock the password file; try again later.\n"));
 		SYSLOG ((LOG_WARN, "can't lock /etc/passwd"));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (!pw_open (O_RDWR)) {
 		fprintf (stderr, _("Cannot open the password file.\n"));
 		pw_unlock ();
 		SYSLOG ((LOG_ERR, "can't open /etc/passwd"));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	/*
@@ -542,7 +559,7 @@ int main (int argc, char **argv)
 		pw_unlock ();
 		fprintf (stderr,
 			 _("%s: %s not found in /etc/passwd\n"), Prog, user);
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	/*
@@ -561,7 +578,7 @@ int main (int argc, char **argv)
 		pw_unlock ();
 		SYSLOG ((LOG_ERR, "error updating passwd entry"));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	/*
@@ -572,18 +589,23 @@ int main (int argc, char **argv)
 		pw_unlock ();
 		SYSLOG ((LOG_ERR, "can't rewrite /etc/passwd"));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (!pw_unlock ()) {
 		fprintf (stderr, _("Cannot unlock the password file.\n"));
 		SYSLOG ((LOG_ERR, "can't unlock /etc/passwd"));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	SYSLOG ((LOG_INFO, "changed user `%s' information", user));
 
 	nscd_flush_cache ("passwd");
 
+#ifdef USE_PAM
+	if (retval == PAM_SUCCESS)
+		pam_end (pamh, PAM_SUCCESS);
+#endif				/* USE_PAM */
+
 	closelog ();
-	exit (0);
+	exit (E_SUCCESS);
 }
