@@ -29,8 +29,8 @@
 
 #include <config.h>
 
-#include "rcsid.h"
-RCSID (PKG_VER "$Id: useradd.c,v 1.75 2005/08/11 16:23:34 kloczek Exp $")
+#ident "$Id: useradd.c,v 1.89 2005/12/15 15:06:28 kloczek Exp $"
+
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -39,19 +39,26 @@ RCSID (PKG_VER "$Id: useradd.c,v 1.75 2005/08/11 16:23:34 kloczek Exp $")
 #include <lastlog.h>
 #include <pwd.h>
 #ifdef USE_PAM
-#include <security/pam_appl.h>
-#include <security/pam_misc.h>
+#include "pam_defs.h"
 #endif				/* USE_PAM */
 #include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <time.h>
-#include "prototypes.h"
-#include "defines.h"
 #include "chkname.h"
-#include "pwauth.h"
+#include "defines.h"
 #include "faillog.h"
+#include "getdef.h"
+#include "groupio.h"
 #include "nscd.h"
+#include "prototypes.h"
+#include "pwauth.h"
+#include "pwio.h"
+#ifdef	SHADOWGRP
+#include "sgroupio.h"
+#endif
+#include "shadowio.h"
+
 #ifndef SKEL_DIR
 #define SKEL_DIR "/etc/skel"
 #endif
@@ -65,6 +72,9 @@ RCSID (PKG_VER "$Id: useradd.c,v 1.75 2005/08/11 16:23:34 kloczek Exp $")
 #ifndef LASTLOG_FILE
 #define LASTLOG_FILE "/var/log/lastlog"
 #endif
+/*
+ * Global variables
+ */
 /*
  * These defaults are used if there is no defaults file.
  */
@@ -124,16 +134,6 @@ extern int optind;
 
 static int home_added;
 
-#include "groupio.h"
-
-#ifdef	SHADOWGRP
-#include "sgroupio.h"
-#endif
-
-#include "pwio.h"
-#include "shadowio.h"
-#include "getdef.h"
-
 /*
  * exit status values
  */
@@ -185,17 +185,19 @@ static void create_mail (void);
 /*
  * fail_exit - undo as much as possible
  */
-
 static void fail_exit (int code)
 {
 	if (home_added)
 		rmdir (user_home);
 
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding user", user_name, -1,
+		      0);
+#endif
 	SYSLOG ((LOG_INFO, "failed adding user `%s', data deleted", user_name));
 	exit (code);
 }
 
-
 static struct group *getgr_nam_gid (const char *name)
 {
 	gid_t gid;
@@ -208,7 +210,6 @@ static struct group *getgr_nam_gid (const char *name)
 	return getgrnam (name);
 }
 
-
 static long get_number (const char *cp)
 {
 	long val;
@@ -244,7 +245,6 @@ static uid_t get_uid (const char *cp)
  *	various values from the file, or uses built-in default values if the
  *	file does not exist.
  */
-
 static void get_defaults (void)
 {
 	FILE *fp;
@@ -263,7 +263,6 @@ static void get_defaults (void)
 	 * Read the file a line at a time. Only the lines that have relevant
 	 * values are used, everything else can be ignored.
 	 */
-
 	while (fgets (buf, sizeof buf, fp)) {
 		if ((cp = strrchr (buf, '\n')))
 			*cp = '\0';
@@ -276,7 +275,6 @@ static void get_defaults (void)
 		/*
 		 * Primary GROUP identifier
 		 */
-
 		if (MATCH (buf, DGROUP)) {
 			unsigned int val = (unsigned int) strtoul (cp, &ep, 10);
 
@@ -353,14 +351,12 @@ static void get_defaults (void)
 	}
 }
 
-
 /*
  * show_defaults - show the contents of the defaults file
  *
  *	show_defaults() displays the values that are used from the default
  *	file and the built-in values.
  */
-
 static void show_defaults (void)
 {
 	printf ("GROUP=%u\n", (unsigned int) def_group);
@@ -379,7 +375,6 @@ static void show_defaults (void)
  *	are currently set. Duplicated lines are pruned, missing lines are
  *	added, and unrecognized lines are copied as is.
  */
-
 static int set_defaults (void)
 {
 	FILE *ifp;
@@ -399,7 +394,6 @@ static int set_defaults (void)
 	/*
 	 * Create a temporary file to copy the new output to.
 	 */
-
 	if ((ofd = mkstemp (new_file)) == -1) {
 		fprintf (stderr,
 			 _("%s: cannot create new defaults file\n"), Prog);
@@ -417,7 +411,6 @@ static int set_defaults (void)
 	 * temporary file, using any new values. Each line is checked
 	 * to insure that it is not output more than once.
 	 */
-
 	if (!(ifp = fopen (def_file, "r"))) {
 		fprintf (ofp, "# useradd defaults file\n");
 		goto skip;
@@ -461,7 +454,6 @@ static int set_defaults (void)
 	 * causes new values to be added to a file which did not previously
 	 * have an entry for that value.
 	 */
-
 	if (!out_group)
 		fprintf (ofp, DGROUP "%u\n", (unsigned int) def_group);
 	if (!out_home)
@@ -482,7 +474,6 @@ static int set_defaults (void)
 	 * Flush and close the file. Check for errors to make certain
 	 * the new file is intact.
 	 */
-
 	fflush (ofp);
 	if (ferror (ofp) || fclose (ofp)) {
 		unlink (new_file);
@@ -492,7 +483,6 @@ static int set_defaults (void)
 	/*
 	 * Rename the current default file to its backup name.
 	 */
-
 	snprintf (buf, sizeof buf, "%s-", def_file);
 	if (rename (def_file, buf) && errno != ENOENT) {
 		snprintf (buf, sizeof buf, _("%s: rename: %s"), Prog, def_file);
@@ -504,12 +494,15 @@ static int set_defaults (void)
 	/*
 	 * Rename the new default file to its correct name.
 	 */
-
 	if (rename (new_file, def_file)) {
 		snprintf (buf, sizeof buf, _("%s: rename: %s"), Prog, new_file);
 		perror (buf);
 		return -1;
 	}
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "changing user defaults",
+		      NULL, -1, 1);
+#endif
 	SYSLOG ((LOG_INFO,
 		 "useradd defaults: GROUP=%u, HOME=%s, SHELL=%s, INACTIVE=%ld, "
 		 "EXPIRE=%s, SKEL=%s, CREATE_MAIL_SPOOL=%s",
@@ -526,7 +519,6 @@ static int set_defaults (void)
  *	converts it to a NULL-terminated array. Any unknown group
  *	names are reported as errors.
  */
-
 static int get_groups (char *list)
 {
 	char *cp;
@@ -534,12 +526,6 @@ static int get_groups (char *list)
 	int errors = 0;
 	int ngroups = 0;
 
-	/*
-	 * Initialize the list to be empty
-	 */
-
-	user_groups[0] = (char *) 0;
-
 	if (!*list)
 		return 0;
 
@@ -548,12 +534,10 @@ static int get_groups (char *list)
 	 * each name and look it up. A mix of numerical and string
 	 * values for group identifiers is permitted.
 	 */
-
 	do {
 		/*
 		 * Strip off a single name from the list
 		 */
-
 		if ((cp = strchr (list, ',')))
 			*cp++ = '\0';
 
@@ -561,14 +545,12 @@ static int get_groups (char *list)
 		 * Names starting with digits are treated as numerical
 		 * GID values, otherwise the string is looked up as is.
 		 */
-
 		grp = getgr_nam_gid (list);
 
 		/*
 		 * There must be a match, either by GID value or by
 		 * string name.
 		 */
-
 		if (!grp) {
 			fprintf (stderr, _("%s: unknown group %s\n"),
 				 Prog, list);
@@ -588,7 +570,6 @@ static int get_groups (char *list)
 		 * Don't add this group if they are an NIS group. Tell
 		 * the user to go to the server for this group.
 		 */
-
 		if (__isgrNIS ()) {
 			fprintf (stderr,
 				 _("%s: group `%s' is a NIS group.\n"),
@@ -608,7 +589,6 @@ static int get_groups (char *list)
 		/*
 		 * Add the group name to the user's list of groups.
 		 */
-
 		user_groups[ngroups++] = xstrdup (grp->gr_name);
 	} while (list);
 
@@ -617,7 +597,6 @@ static int get_groups (char *list)
 	/*
 	 * Any errors in finding group names are fatal
 	 */
-
 	if (errors)
 		return -1;
 
@@ -627,7 +606,6 @@ static int get_groups (char *list)
 /*
  * usage - display usage message and exit
  */
-
 static void usage (void)
 {
 	fprintf (stderr, _("Usage: useradd [options] LOGIN\n"
@@ -646,7 +624,7 @@ static void usage (void)
 			   "  -G, --groups GROUPS		list of supplementary groups for the new\n"
 			   "				user account\n"
 			   "  -h, --help			display this help message and exit\n"
-			   "  -k, --skel SKEL_DIR 		specify an alternative skel directory\n"
+			   "  -k, --skel SKEL_DIR		specify an alternative skel directory\n"
 			   "  -K, --key KEY=VALUE		overrides /etc/login.defs defaults\n"
 			   "  -m, --create-home		create home directory for the new user\n"
 			   "				account\n"
@@ -665,7 +643,6 @@ static void usage (void)
  *	new_pwent() takes all of the values that have been entered and
  *	fills in a (struct passwd) with them.
  */
-
 static void new_pwent (struct passwd *pwent)
 {
 	memzero (pwent, sizeof *pwent);
@@ -696,7 +673,6 @@ static long scale_age (long x)
  *	new_spent() takes all of the values that have been entered and
  *	fills in a (struct spwd) with them.
  */
-
 static void new_spent (struct spwd *spent)
 {
 	memzero (spent, sizeof *spent);
@@ -717,7 +693,6 @@ static void new_spent (struct spwd *spent)
  *	grp_update() takes the secondary group set given in user_groups
  *	and adds the user to each group given by that set.
  */
-
 static void grp_update (void)
 {
 	const struct group *grp;
@@ -729,43 +704,41 @@ static void grp_update (void)
 #endif
 
 	/*
-	 * Lock and open the group file. This will load all of the group
-	 * entries.
+	 * Test for unique entries of user_groups in /etc/group
+	 * pvrabec@redhat.com
 	 */
+	char **user_groups_tmp = user_groups;
 
-	if (!gr_lock ()) {
-		fprintf (stderr, _("%s: error locking group file\n"), Prog);
-		fail_exit (E_GRP_UPDATE);
+	while (*user_groups_tmp) {
+		int count = 0;
+
+		for (gr_rewind (), grp = gr_next (); grp && count < 2;
+		     grp = gr_next ()) {
+			if (strcmp (*user_groups_tmp, grp->gr_name) == 0) {
+				count++;
+			}
+		}
+		if (count > 1) {
+			fprintf (stderr,
+				 "%s: error not unique group names in group file\n",
+				 Prog);
+			fail_exit (E_GRP_UPDATE);
+		}
+		user_groups_tmp++;
 	}
-	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: error opening group file\n"), Prog);
-		fail_exit (E_GRP_UPDATE);
-	}
-#ifdef	SHADOWGRP
-	if (is_shadow_grp && !sgr_lock ()) {
-		fprintf (stderr,
-			 _("%s: error locking shadow group file\n"), Prog);
-		fail_exit (E_GRP_UPDATE);
-	}
-	if (is_shadow_grp && !sgr_open (O_RDWR)) {
-		fprintf (stderr,
-			 _("%s: error opening shadow group file\n"), Prog);
-		fail_exit (E_GRP_UPDATE);
-	}
-#endif
+
+	/* Locking and opening of the group files moved to open_files() --gafton */
 
 	/*
 	 * Scan through the entire group file looking for the groups that
 	 * the user is a member of.
 	 */
-
 	for (gr_rewind (), grp = gr_next (); grp; grp = gr_next ()) {
 
 		/*
 		 * See if the user specified this group as one of their
 		 * concurrent groups.
 		 */
-
 		if (!is_on_list (user_groups, grp->gr_name))
 			continue;
 
@@ -773,7 +746,6 @@ static void grp_update (void)
 		 * Make a copy - gr_update() will free() everything
 		 * from the old entry, and we need it later.
 		 */
-
 		ngrp = __gr_dup (grp);
 		if (!ngrp) {
 			fail_exit (E_GRP_UPDATE);	/* XXX */
@@ -783,13 +755,16 @@ static void grp_update (void)
 		 * Add the username to the list of group members and
 		 * update the group entry to reflect the change.
 		 */
-
 		ngrp->gr_mem = add_list (ngrp->gr_mem, user_name);
 		if (!gr_update (ngrp)) {
 			fprintf (stderr,
-				 "%s: error adding new group entry\n", Prog);
+				 _("%s: error adding new group entry\n"), Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "adding user to group", user_name, -1, 1);
+#endif
 		SYSLOG ((LOG_INFO, "add `%s' to group `%s'",
 			 user_name, ngrp->gr_name));
 	}
@@ -803,14 +778,12 @@ static void grp_update (void)
 	 * that the user is a member of. The administrative list isn't
 	 * modified.
 	 */
-
 	for (sgr_rewind (), sgrp = sgr_next (); sgrp; sgrp = sgr_next ()) {
 
 		/*
 		 * See if the user specified this group as one of their
 		 * concurrent groups.
 		 */
-
 		if (!gr_locate (sgrp->sg_name))
 			continue;
 
@@ -821,7 +794,6 @@ static void grp_update (void)
 		 * Make a copy - sgr_update() will free() everything
 		 * from the old entry, and we need it later.
 		 */
-
 		nsgrp = __sgr_dup (sgrp);
 		if (!nsgrp) {
 			fail_exit (E_GRP_UPDATE);	/* XXX */
@@ -831,13 +803,16 @@ static void grp_update (void)
 		 * Add the username to the list of group members and
 		 * update the group entry to reflect the change.
 		 */
-
 		nsgrp->sg_mem = add_list (nsgrp->sg_mem, user_name);
 		if (!sgr_update (nsgrp)) {
 			fprintf (stderr,
 				 _("%s: error adding new group entry\n"), Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "adding user to shadow group", user_name, -1, 1);
+#endif
 		SYSLOG ((LOG_INFO, "add `%s' to shadow group `%s'",
 			 user_name, nsgrp->sg_name));
 	}
@@ -851,20 +826,18 @@ static void grp_update (void)
  *	file, or checks the given user ID against the existing ones for
  *	uniqueness.
  */
-
 static void find_new_uid (void)
 {
 	const struct passwd *pwd;
 	uid_t uid_min, uid_max;
 
-	uid_min = getdef_unum ("UID_MIN", 100);
+	uid_min = getdef_unum ("UID_MIN", 1000);
 	uid_max = getdef_unum ("UID_MAX", 60000);
 
 	/*
 	 * Start with some UID value if the user didn't provide us with
 	 * one already.
 	 */
-
 	if (!uflg)
 		user_id = uid_min;
 
@@ -873,7 +846,6 @@ static void find_new_uid (void)
 	 * UID (if the user specified one with -u) or looking for the
 	 * largest unused value.
 	 */
-
 #ifdef NO_GETPWENT
 	pw_rewind ();
 	while ((pwd = pw_next ())) {
@@ -884,11 +856,19 @@ static void find_new_uid (void)
 		if (strcmp (user_name, pwd->pw_name) == 0) {
 			fprintf (stderr, _("%s: name %s is not unique\n"),
 				 Prog, user_name);
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding user",
+				      user_name, user_id, 0);
+#endif
 			exit (E_NAME_IN_USE);
 		}
 		if (uflg && user_id == pwd->pw_uid) {
 			fprintf (stderr, _("%s: UID %u is not unique\n"),
 				 Prog, (unsigned int) user_id);
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding user",
+				      user_name, user_id, 0);
+#endif
 			exit (E_UID_IN_USE);
 		}
 		if (!uflg && pwd->pw_uid >= user_id) {
@@ -897,6 +877,7 @@ static void find_new_uid (void)
 			user_id = pwd->pw_uid + 1;
 		}
 	}
+
 	/*
 	 * If a user with UID equal to UID_MAX exists, the above algorithm
 	 * will give us UID_MAX+1 even if not unique. Search for the first
@@ -923,6 +904,80 @@ static void find_new_uid (void)
 	}
 }
 
+ /*
+  * find_new_gid - find the next available GID
+  *
+  *     find_new_gid() locates the next highest unused GID in the group
+  *     file, or checks the given group ID against the existing ones for
+  *     uniqueness.
+  */
+
+static void find_new_gid ()
+{
+	const struct group *grp;
+	gid_t gid_min, gid_max;
+
+	gid_min = getdef_num ("GID_MIN", 500);
+	gid_max = getdef_num ("GID_MAX", 60000);
+
+	/*
+	 * Start with some GID value if the user didn't provide us with
+	 * one already.
+	 */
+	user_gid = gid_min;
+
+	/*
+	 * Search the entire group file, either looking for this
+	 * GID (if the user specified one with -g) or looking for the
+	 * largest unused value.
+	 */
+#ifdef NO_GETGRENT
+	gr_rewind ();
+	while ((grp = gr_next ()))
+#else
+	setgrent ();
+	while ((grp = getgrent ()))
+#endif
+	{
+		if (strcmp (user_name, grp->gr_name) == 0) {
+			user_gid = grp->gr_gid;
+			return;
+		}
+		if (grp->gr_gid >= user_gid) {
+			if (grp->gr_gid > gid_max)
+				continue;
+			user_gid = grp->gr_gid + 1;
+		}
+	}
+#ifndef NO_GETGRENT		/* glibc does have this, so ... */
+	/* A quick test gets here: if the UID is available
+	 * as a GID, go ahead and use it */
+	if (!getgrgid (user_id)) {
+		user_gid = user_id;
+		return;
+	}
+#endif
+	if (user_gid == gid_max + 1) {
+		for (user_gid = gid_min; user_gid < gid_max; user_gid++) {
+#ifdef NO_GETGRENT
+			gr_rewind ();
+			while ((grp = gr_next ()) && grp->gr_gid != user_gid);
+			if (!grp)
+				break;
+#else
+			if (!getgrgid (user_gid))
+				break;
+#endif
+		}
+		if (user_gid == gid_max) {
+			fprintf (stderr,
+				 "%s: can't get unique gid (run out of GIDs)\n",
+				 Prog);
+			fail_exit (4);
+		}
+	}
+}
+
 /*
  * process_flags - perform command line argument setting
  *
@@ -930,7 +985,6 @@ static void find_new_uid (void)
  *	the values that the user will be created with accordingly. The
  *	values are checked for sanity.
  */
-
 static void process_flags (int argc, char **argv)
 {
 	const struct group *grp;
@@ -1140,6 +1194,7 @@ static void process_flags (int argc, char **argv)
 			anyflag++;
 		}
 	}
+
 	/*
 	 * Certain options are only valid in combination with others.
 	 * Check it here so that they can be specified in any order.
@@ -1167,6 +1222,10 @@ static void process_flags (int argc, char **argv)
 				 _
 				 ("%s: invalid user name '%s'\n"),
 				 Prog, user_name);
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding user",
+				      user_name, -1, 0);
+#endif
 			exit (E_BAD_ARG);
 		}
 		if (!dflg) {
@@ -1195,7 +1254,6 @@ static void process_flags (int argc, char **argv)
  *	close_files() closes all of the files that were opened for this
  *	new user. This causes any modified entries to be written out.
  */
-
 static void close_files (void)
 {
 	if (!pw_close ()) {
@@ -1236,33 +1294,152 @@ static void close_files (void)
  *
  *	open_files() opens the two password files.
  */
-
 static void open_files (void)
 {
 	if (!pw_lock ()) {
 		fprintf (stderr, _("%s: unable to lock password file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "locking password file", user_name, user_id, 0);
+#endif
 		exit (E_PW_UPDATE);
 	}
 	if (!pw_open (O_RDWR)) {
 		fprintf (stderr, _("%s: unable to open password file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "opening password file", user_name, user_id, 0);
+#endif
 		pw_unlock ();
 		exit (E_PW_UPDATE);
 	}
 	if (is_shadow_pwd && !spw_lock ()) {
 		fprintf (stderr,
 			 _("%s: cannot lock shadow password file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "locking shadow password file", user_name,
+			      user_id, 0);
+#endif
 		pw_unlock ();
 		exit (E_PW_UPDATE);
 	}
 	if (is_shadow_pwd && !spw_open (O_RDWR)) {
 		fprintf (stderr,
 			 _("%s: cannot open shadow password file\n"), Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "opening shadow password file", user_name,
+			      user_id, 0);
+#endif
 		spw_unlock ();
 		pw_unlock ();
 		exit (E_PW_UPDATE);
 	}
+	/*
+	 * Lock and open the group file.
+	 */
+
+	if (!gr_lock ()) {
+		fprintf (stderr, _("%s: error locking group file\n"), Prog);
+		fail_exit (E_GRP_UPDATE);
+	}
+	if (!gr_open (O_RDWR)) {
+		fprintf (stderr, _("%s: error opening group file\n"), Prog);
+		fail_exit (E_GRP_UPDATE);
+	}
+#ifdef  SHADOWGRP
+	if (is_shadow_grp && !sgr_lock ()) {
+		fprintf (stderr,
+			 _("%s: error locking shadow group file\n"), Prog);
+		fail_exit (E_GRP_UPDATE);
+	}
+	if (is_shadow_grp && !sgr_open (O_RDWR)) {
+		fprintf (stderr,
+			 _("%s: error opening shadow group file\n"), Prog);
+		fail_exit (E_GRP_UPDATE);
+	}
+#endif
 }
 
+static char *empty_list = NULL;
+
+/*
+ * new_grent - initialize the values in a group file entry
+ *
+ *      new_grent() takes all of the values that have been entered and fills
+ *      in a (struct group) with them.
+ */
+
+static void new_grent (struct group *grent)
+{
+	memzero (grent, sizeof *grent);
+	grent->gr_name = (char *) user_name;
+	grent->gr_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
+	grent->gr_gid = user_gid;
+	grent->gr_mem = &empty_list;
+}
+
+#ifdef  SHADOWGRP
+/*
+ * new_sgent - initialize the values in a shadow group file entry
+ *
+ *      new_sgent() takes all of the values that have been entered and fills
+ *      in a (struct sgrp) with them.
+ */
+
+static void new_sgent (struct sgrp *sgent)
+{
+	memzero (sgent, sizeof *sgent);
+	sgent->sg_name = (char *) user_name;
+	sgent->sg_passwd = "!";	/* XXX warning: const */
+	sgent->sg_adm = &empty_list;
+	sgent->sg_mem = &empty_list;
+}
+#endif				/* SHADOWGRP */
+
+
+/*
+ * grp_add - add new group file entries
+ *
+ *      grp_add() writes the new records to the group files.
+ */
+
+static void grp_add (void)
+{
+	struct group grp;
+
+#ifdef  SHADOWGRP
+	struct sgrp sgrp;
+#endif				/* SHADOWGRP */
+
+	/*
+	 * Create the initial entries for this new group.
+	 */
+	new_grent (&grp);
+#ifdef  SHADOWGRP
+	new_sgent (&sgrp);
+#endif				/* SHADOWGRP */
+
+	/*
+	 * Write out the new group file entry.
+	 */
+	if (!gr_update (&grp)) {
+		fprintf (stderr, _("%s: error adding new group entry\n"), Prog);
+		fail_exit (E_GRP_UPDATE);
+	}
+#ifdef  SHADOWGRP
+	/*
+	 * Write out the new shadow group entries as well.
+	 */
+	if (is_shadow_grp && !sgr_update (&sgrp)) {
+		fprintf (stderr, _("%s: error adding new group entry\n"), Prog);
+		fail_exit (E_GRP_UPDATE);
+	}
+#endif				/* SHADOWGRP */
+	SYSLOG ((LOG_INFO, "new group: name=%s, GID=%u", user_name, user_gid));
+	do_grp_update++;
+}
 
 static void faillog_reset (uid_t uid)
 {
@@ -1298,7 +1475,6 @@ static void lastlog_reset (uid_t uid)
  *	usr_update() creates the password file entries for this user
  *	and will update the group entries if required.
  */
-
 static void usr_update (void)
 {
 	struct passwd pwent;
@@ -1311,7 +1487,6 @@ static void usr_update (void)
 	 * Fill in the password structure with any new fields, making
 	 * copies of strings.
 	 */
-
 	new_pwent (&pwent);
 	new_spent (&spent);
 
@@ -1319,7 +1494,6 @@ static void usr_update (void)
 	 * Create a syslog entry. We need to do this now in case anything
 	 * happens so we know what we were trying to accomplish.
 	 */
-
 	SYSLOG ((LOG_INFO,
 		 "new user: name=%s, UID=%u, GID=%u, home=%s, shell=%s",
 		 user_name, (unsigned int) user_id,
@@ -1331,7 +1505,6 @@ static void usr_update (void)
 	 * no user with this UID exists yet (entries for shared UIDs
 	 * are left unchanged).  --marekm
 	 */
-
 	if (!getpwuid (user_id)) {
 		faillog_reset (user_id);
 		lastlog_reset (user_id);
@@ -1340,7 +1513,6 @@ static void usr_update (void)
 	/*
 	 * Put the new (struct passwd) in the table.
 	 */
-
 	if (!pw_update (&pwent)) {
 		fprintf (stderr,
 			 _("%s: error adding new password entry\n"), Prog);
@@ -1350,19 +1522,25 @@ static void usr_update (void)
 	/*
 	 * Put the new (struct spwd) in the table.
 	 */
-
 	if (is_shadow_pwd && !spw_update (&spent)) {
 		fprintf (stderr,
 			 _
 			 ("%s: error adding new shadow password entry\n"),
 			 Prog);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "adding shadow password", user_name, user_id, 0);
+#endif
 		exit (E_PW_UPDATE);
 	}
+#ifdef WITH_AUDIT
+	audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding user", user_name,
+		      user_id, 1);
+#endif
 
 	/*
 	 * Do any group file updates for this user.
 	 */
-
 	if (do_grp_update)
 		grp_update ();
 }
@@ -1374,7 +1552,6 @@ static void usr_update (void)
  *	already exist. It will be created mode 755 owned by the user
  *	with the user's default group.
  */
-
 static void create_home (void)
 {
 	if (access (user_home, F_OK)) {
@@ -1389,6 +1566,10 @@ static void create_home (void)
 		chown (user_home, user_id, user_gid);
 		chmod (user_home, 0777 & ~getdef_num ("UMASK", 022));
 		home_added++;
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+			      "adding home directory", user_name, user_id, 1);
+#endif
 	}
 }
 
@@ -1444,17 +1625,9 @@ static void create_mail (void)
 	}
 }
 
-#ifdef USE_PAM
-static struct pam_conv conv = {
-	misc_conv,
-	NULL
-};
-#endif				/* USE_PAM */
-
 /*
  * main - useradd command
  */
-
 int main (int argc, char **argv)
 {
 #ifdef USE_PAM
@@ -1463,10 +1636,13 @@ int main (int argc, char **argv)
 	int retval;
 #endif
 
+#ifdef WITH_AUDIT
+	audit_help_open ();
+#endif
+
 	/*
 	 * Get my name so that I can use it to report errors.
 	 */
-
 	Prog = Basename (argv[0]);
 
 	setlocale (LC_ALL, "");
@@ -1477,6 +1653,11 @@ int main (int argc, char **argv)
 
 	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
 	user_groups = malloc ((1 + sys_ngroups) * sizeof (char *));
+	/*
+	 * Initialize the list to be empty
+	 */
+	user_groups[0] = (char *) 0;
+
 
 	is_shadow_pwd = spw_file_present ();
 #ifdef SHADOWGRP
@@ -1523,7 +1704,6 @@ int main (int argc, char **argv)
 	 * See if we are messing with the defaults file, or creating
 	 * a new user.
 	 */
-
 	if (Dflg) {
 		if (gflg || bflg || fflg || eflg || sflg)
 			exit (set_defaults ()? 1 : 0);
@@ -1535,9 +1715,12 @@ int main (int argc, char **argv)
 	/*
 	 * Start with a quick check to see if the user exists.
 	 */
-
 	if (getpwnam (user_name)) {
 		fprintf (stderr, _("%s: user %s exists\n"), Prog, user_name);
+#ifdef WITH_AUDIT
+		audit_logger (AUDIT_USER_CHAUTHTOK, Prog, "adding user",
+			      user_name, -1, 0);
+#endif
 		exit (E_NAME_IN_USE);
 	}
 
@@ -1553,6 +1736,10 @@ int main (int argc, char **argv)
 				 _
 				 ("%s: group %s exists - if you want to add this user to that group, use -g.\n"),
 				 Prog, user_name);
+#ifdef WITH_AUDIT
+			audit_logger (AUDIT_USER_CHAUTHTOK, Prog,
+				      "adding group", user_name, -1, 0);
+#endif
 			exit (E_NAME_IN_USE);
 		}
 	}
@@ -1566,9 +1753,20 @@ int main (int argc, char **argv)
 	 * - flush nscd caches for passwd and group services,
 	 * - then close and update the files.
 	 */
-
 	open_files ();
 
+	/* first, seek for a valid uid to use for this user.
+	 * We do this because later we can use the uid we found as
+	 * gid too ... --gafton */
+	find_new_uid ();
+
+	/* do we have to add a group for that user? This is why we need to
+	 * open the group files in the open_files() function  --gafton */
+	if (!(nflg || gflg)) {
+		find_new_gid ();
+		grp_add ();
+	}
+
 	usr_update ();
 
 	if (mflg) {
@@ -1579,7 +1777,8 @@ int main (int argc, char **argv)
 			fprintf (stderr,
 				 _
 				 ("%s: warning: the home directory already exists.\n"
-				  "Not copying any file from skel directory into it.\n"), Prog);
+				  "Not copying any file from skel directory into it.\n"),
+				 Prog);
 
 	} else if (getdef_str ("CREATE_HOME")) {
 		/*
@@ -1608,18 +1807,6 @@ int main (int argc, char **argv)
 	close_files ();
 
 #ifdef USE_PAM
-	if (retval == PAM_SUCCESS) {
-		retval = pam_chauthtok (pamh, 0);
-		if (retval != PAM_SUCCESS) {
-			pam_end (pamh, retval);
-		}
-	}
-
-	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM chauthtok failed\n"), Prog);
-		exit (1);
-	}
-
 	if (retval == PAM_SUCCESS)
 		pam_end (pamh, PAM_SUCCESS);
 #endif				/* USE_PAM */
